#!/usr/bin/python
# client.py - routines for snake client programs

import snake.tree
import xmlrpclib

# Minimum snake XMLRPC API version
minimum_version = 1

def connect(server_url):
    '''Connect to the given server url. Returns a ServerProxy object.'''
    return xmlrpclib.ServerProxy(server_url)

def check_server(server_url):
    '''Check the server given by the URL.'''
    try:
        server = xmlrpclib.ServerProxy(server_url)
        methods = server.system.listMethods()
    except:
        return False

    if 'snake.version' not in methods:
        return False
    if server.snake.version() < minimum_version:
        return False

    return True

def treedict(server,**args):
    '''Return a dict of trees (using tree.id as the keys) that match the args
    given. Args will be passed to the server tree.list method.'''
    td = [(t['id'],snake.tree.Tree(dict=t)) for t in server.tree.list(args)]
    return dict(td)
