#!/usr/bin/python
# uri.py - misc. routines for handing URIs to trees etc.

import os, urllib2, urlparse
# Hack: make urlparse handle nfs the way we expect
urlparse.uses_netloc.append('nfs')
from snake.tree import Tree

# URI errors are basically a failure to communicate with a remote machine, so
# this is subclassed from IOError. urllib2.URLError is also an IOError subclass.
class URIError(IOError):
    pass

def uriopen(uri):
    '''Open a URL and return a file-like object.'''
    fd = None
    u = parse(uri)
    if u.scheme in ('http','https','ftp'):
        try:
            fd = urllib2.urlopen(uri)
        except IOError, e:
            raise URIError, "urllib error: %s" % e
        if uri.startswith('ftp') and not fd.info():
            raise URIError, "urllib error: file not found"
    else:
        raise URIError, "Unimplemented scheme %s" % u.scheme
    return fd

def fetchuri(uri,target,overwrite=True):
    '''Fetches the file given by the URI into the target dir.
    If overwrite is false, it will append ".1" (or similar) to the filename
    if it already exists.'''
    targetfile = os.path.join(target,os.path.basename(uri))
    if not overwrite and os.path.exists(targetfile):
        count = 1
        newfile = "%s.%i" % (targetfile,count)
        while os.path.exists(newfile):
            count = count + 1
            newfile = "%s.%i" % (targetfile,count)
        targetfile = newfile
    data = uriopen(uri)
    f = open(targetfile,"w")
    f.write(data.read())
    data.close()
    f.close()
    return targetfile

def parse(uri):
    '''Returns true if uri is a usable SNAKE uri'''
    uri_obj = urlparse.urlsplit(uri)

    if isinstance(uri_obj, tuple):
        '''Older python returns a tuple, let's fake a class'''
        class UriDummyType: 
            def geturl(self):
                return urlparse.urlunsplit((self.scheme, self.hostname, self.path, self.query, self.fragment))
        u = UriDummyType()
        (u.scheme, u.hostname, u.path, u.query, u.fragment) = uri_obj
        uri_obj = u

    if not (uri_obj.scheme in ('http','https','ftp') and \
            uri_obj.path and uri_obj.hostname):
        raise URIError, 'Unusable URI: %s' % uri
    return uri_obj

def treeinfo(uri):
    '''Returns a file-like object for the .treeinfo in the given URI'''
    return uriopen(os.path.join(uri,".treeinfo"))

def discinfo(uri):
    '''Returns a file-like object for the .discinfo in the given URI'''
    return uriopen(os.path.join(uri,".discinfo"))

def uri_to_tree(uri):
    '''Return a Tree object representing the tree at the given URI'''
    t = Tree()
    try:
        t.parsetreeinfo(treeinfo(uri))
    except URIError:
        t.parsediscinfo(discinfo(uri))
    return t

def check(uri):
    '''Check the validity of a given URI'''
    try:
        t = treeinfo(uri)
    except URIError:
        try:
            t = discinfo(uri)
        except URIError:
            return False
    t.close()
    return True
